package com.huimv.wine.ws;

import com.alibaba.fastjson.JSON;
import com.huimv.wine.common.EventWsErrUtil;
import com.huimv.wine.common.TimeUtil;
import com.huimv.wine.config.VipCofing;
import com.huimv.wine.entity.Advertise;
import com.huimv.wine.entity.Device;
import com.huimv.wine.entity.Params;
import com.huimv.wine.entity.Wine;
import com.huimv.wine.entity.vo.QrcodeParam;
import com.huimv.wine.entity.vo.TradeRedis;
import com.huimv.wine.entity.vo.WsEvent;
import com.huimv.wine.mapper.AdvertiseMapper;
import com.huimv.wine.mapper.DeviceMapper;
import com.huimv.wine.mapper.ParamsMapper;
import com.huimv.wine.mapper.WineMapper;
import com.huimv.wine.utils.SpringContextUtil;
import com.huimv.wine.utils.WebsocketSellerUtil;
import lombok.extern.log4j.Log4j2;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;
import org.springframework.util.ObjectUtils;
import org.springframework.web.socket.handler.TextWebSocketHandler;

import javax.websocket.*;
import javax.websocket.server.PathParam;
import javax.websocket.server.ServerEndpoint;
import java.io.IOException;
import java.time.Duration;
import java.util.*;

/**
 * websocket接口处理类
 */
@Component
@ServerEndpoint(value = "/seller/{seq}")
@Log4j2
public class SellerController extends TextWebSocketHandler {


    /**
     * 连接事件，加入注解
     */
    @OnOpen
    public void onOpen(@PathParam(value = "seq") String seq, Session session) {
        String message = "[" + seq + "]加入聊天室！！";
        System.out.println(message);

        //需要用这种方式注入
        DeviceMapper deviceMapper = SpringContextUtil.getBean(DeviceMapper.class);
        Device device =  deviceMapper.getByDeviceId(seq);
        if (ObjectUtils.isEmpty(device)){
            device = new Device();
            device.setId(seq);
            deviceMapper.insert(device);
        }else {
            device.setLast(new Date());
            deviceMapper.updateDeviceById(device);
        }
        WebsocketSellerUtil.addSession(seq, session);
        //初始化
        infoOfDevice(device,session,deviceMapper);
    }

    /**
     * 连接事件，加入注解
     * 用户断开链接
     */
    @OnClose
    public void onClose(@PathParam(value = "seq") String seq, Session session) {

        // 删除映射关系
        WebsocketSellerUtil.removeSession(seq);

    }


    @OnMessage
    public void onMessage(@PathParam(value = "seq") String seq,  Session session, String message) {
//        RemoteEndpoint.Async asyncRemote = session.getAsyncRemote();
        if ("".equals(seq)){
            WebsocketSellerUtil.sendMessage(session, EventWsErrUtil.getWsErr("device seq is required"));
            return;
        }
        //获取设备
        DeviceMapper deviceMapper = SpringContextUtil.getBean(DeviceMapper.class);
        Device device =  deviceMapper.getByDeviceId(seq);
        if (ObjectUtils.isEmpty(device)){
            WebsocketSellerUtil.sendMessage(session, EventWsErrUtil.getWsErr("device not fount"));
            return;
        }
        if (ObjectUtils.isEmpty(message)){
            WebsocketSellerUtil.sendMessage(session, EventWsErrUtil.getWsErr("data not fount"));
            return;
        }


        WsEvent wsEvent = JSON.parseObject(message, WsEvent.class);
        String event = wsEvent.getEvent();
        Object data = wsEvent.getData();

        switch (event){
            case "pin":
                keepAlive(session,wsEvent.getData());
                break;
            case "getQrcode":
                getQrcode(device, session, data);
            default:
                WebsocketSellerUtil.sendMessage(session,new WsEvent("__Error_Event__","unrecognized event"));
        }
        //下发数据
//        asyncRemote.sendText(message);

//        WebsocketSellerUtil.sendMessage(session,message);
        // 直接广播

    }

    //生成支付码
    private void getQrcode(Device device, Session session, Object data) {
        WineMapper wineMapper = SpringContextUtil.getBean(WineMapper.class);
        RedisTemplate redisTemplate = SpringContextUtil.getBean(RedisTemplate.class);

        QrcodeParam qrcodeParam =   (QrcodeParam) data;
        int weight = qrcodeParam.getWeight();
        int id = qrcodeParam.getId();
        int cash = qrcodeParam.getCash();
        int cell = qrcodeParam.getCell();

        Wine wine = wineMapper.selectWineById(id);
        if (ObjectUtils.isEmpty(wine)){
            WebsocketSellerUtil.sendMessage(session,EventWsErrUtil.getWsErr(String.format("no such wine: %d", qrcodeParam.getId())));
            return;
        }
        int nowCash = weight * wine.getPrice();
        if (nowCash != cash){
            WebsocketSellerUtil.sendMessage(session,EventWsErrUtil.getWsErr(String.format("got a wrong cash[you: %d, real: %d]",cash,nowCash)));
            return;
        }
        String trade = id+TimeUtil.getNowStringTime();
        String store = JSON.toJSONString(new TradeRedis(device.getId(),cell,id,weight,cash));
        redisTemplate.opsForValue().set(String.format("WxPay_%s", trade),store, Duration.ofSeconds(1800));

    }


    private void keepAlive(Session session, Object data) {
        WebsocketSellerUtil.sendMessage(session,new WsEvent("advResult",data));
    }


    public void   infoOfDevice(Device device,Session session, DeviceMapper deviceMapper) {

        WineMapper wineMapper = SpringContextUtil.getBean(WineMapper.class);
        AdvertiseMapper advertiseMapper = SpringContextUtil.getBean(AdvertiseMapper.class);
        ParamsMapper paramsMapper = SpringContextUtil.getBean(ParamsMapper.class);
        VipCofing vipCofing = SpringContextUtil.getBean(VipCofing.class);

        //初始化酒列表
        List<Wine> wineList = new ArrayList<Wine>();
        wineList.add(wineMapper.selectWineById(device.getWine1()));
        wineList.add(wineMapper.selectWineById(device.getWine2()));
        wineList.add(wineMapper.selectWineById(device.getWine3()));
        wineList.add(wineMapper.selectWineById(device.getWine4()));
        WebsocketSellerUtil.sendMessage(session,new WsEvent("wineResult",wineList));

        //初始化广告列表
        List<Advertise> advertiseList  = advertiseMapper.listAll();
        WebsocketSellerUtil.sendMessage(session,new WsEvent("advResult",advertiseList));

        //初始化配置
        List<Params> paramsList  = paramsMapper.listAll();
        WebsocketSellerUtil.sendMessage(session,new WsEvent("runParamResult",paramsList));

        //生成二维码
//        String codeBase = CodeImgUtil.generateQRCode(vipCofing.getVipScanApi() + "&device=" + device.getId() + "&callback=" + vipCofing.getVipScanApi(), 200, 200);
//        WebsocketSellerUtil.sendMessage(session,new WsEvent("vipQrcodeResult",codeBase));
        WebsocketSellerUtil.sendMessage(session,new WsEvent("initFinish",null));
    }

    /**
     * 处理用户活连接异常
     *
     * @param session
     * @param throwable
     */
    @OnError
    public void onError(Session session, Throwable throwable) {
        try {
            session.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
        throwable.printStackTrace();
    }


}
