package com.huimv.wine.ws.admin;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.huimv.wine.common.EventWsErrUtil;
import com.huimv.wine.entity.Manager;
import com.huimv.wine.entity.Trade;
import com.huimv.wine.entity.vo.QueryAdminVo;
import com.huimv.wine.entity.vo.RankAdminsVo;
import com.huimv.wine.entity.vo.RankDevicesVo;
import com.huimv.wine.entity.vo.WsEvent;
import com.huimv.wine.mapper.ChangeMapper;
import com.huimv.wine.mapper.DeviceMapper;
import com.huimv.wine.mapper.TradeMapper;
import com.huimv.wine.utils.Result;
import com.huimv.wine.utils.WebsocketManagerUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.websocket.Session;
import java.util.ArrayList;
import java.util.List;

@Component
public class AdminDashboard {
    @Autowired
    private TradeMapper tradeMapper;
    @Autowired
    private DeviceMapper deviceMapper;
    @Autowired
    private ChangeMapper changeMapper;

    public void handle(String event, JSONObject data, Session session, Manager manager) {
        switch (event) {
            case "adminDashboardRankDevices":
                rankDevices(session, manager, data);
                break;
            case "adminDashboardRankWines":
                rankWines(session, manager, data);
                break;
            case "adminDashboardRankWorkers":
                rankWorkers(session, manager, data);
                break;
            case "adminDashboardQuerySelf":
                querySelf(session, manager, data);
                break;
            case "adminDashboardQueryDevice":
                queryDevice(session, manager, data);
                break;
            case "adminDashboardQueryWine":
                queryWine(session, manager, data);
                break;
            case "adminDashboardQueryWorker":
                queryWorker(session, manager, data);
                break;
            default:
                WsEvent wsEvent = EventWsErrUtil.getWsErr("unrecognized event");
                WebsocketManagerUtil.sendMessage(session, wsEvent);
                break;
        }
    }

    public void rankDevices(Session session, Manager manager, JSONObject data) {
        if (StringUtils.isBlank(data.getString("start")) || StringUtils.isBlank(data.getString("end"))) {
            WsEvent wsEvent = new WsEvent("adminDashboardRankDevicesRes", Result.fail("参数错误"));
            WebsocketManagerUtil.sendMessage(session, wsEvent);
            return;
        }
        List<RankDevicesVo> rankDevicesVos = deviceMapper.TradeRankDevicesForAdmin(manager.getId(), data.getString("start"), data.getString("end"));
        WsEvent wsEvent = new WsEvent("adminDashboardRankDevicesRes", Result.success(rankDevicesVos));
        WebsocketManagerUtil.sendMessage(session, wsEvent);
        return;
    }

    public void rankWines(Session session, Manager manager, JSONObject data) {
        if (StringUtils.isBlank(data.getString("start")) || StringUtils.isBlank(data.getString("end"))) {
            WsEvent wsEvent = new WsEvent("adminDashboardRankWinesRes", Result.fail("参数错误"));
            WebsocketManagerUtil.sendMessage(session, wsEvent);
            return;
        }
        List<RankAdminsVo> rankAdminsVos = tradeMapper.TradeRankWinesForAdmin(manager.getId(), data.getString("start"), data.getString("end"));
        WsEvent wsEvent = new WsEvent("adminDashboardRankWinesRes", Result.success(rankAdminsVos));
        WebsocketManagerUtil.sendMessage(session, wsEvent);
        return;
    }

    public void rankWorkers(Session session, Manager manager, JSONObject data) {
        if (StringUtils.isBlank(data.getString("start")) || StringUtils.isBlank(data.getString("end"))) {
            WsEvent wsEvent = new WsEvent("adminDashboardRankWorkersRes", Result.fail("参数错误"));
            WebsocketManagerUtil.sendMessage(session, wsEvent);
            return;
        }
        List<RankAdminsVo> rankAdminsVos = changeMapper.ChangeRankWorkersForAdmin(manager.getId(), data.getString("start"), data.getString("end"));
        WsEvent wsEvent = new WsEvent("adminDashboardRankWorkersRes", Result.success(rankAdminsVos));
        WebsocketManagerUtil.sendMessage(session, wsEvent);
        return;
    }

    public void querySelf(Session session, Manager manager, JSONObject data) {
        if (StringUtils.isBlank(data.getString("id")) || data.getInteger("type") == null) {
            WsEvent wsEvent = new WsEvent("adminDashboardQuerySelfRes", Result.fail("参数错误"));
            WebsocketManagerUtil.sendMessage(session, wsEvent);
            return;
        }
        Integer type = data.getInteger("type");
        if (0 == type) {
            QueryWrapper<Trade> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("manager", data.getString("id"));
            List<QueryAdminVo> queryAdmins = tradeMapper.queryAdmins(queryWrapper);
            WsEvent wsEvent = new WsEvent("adminDashboardQuerySelfRes", Result.success(queryAdmins));
            WebsocketManagerUtil.sendMessage(session, wsEvent);
        } else {
            List<QueryAdminVo> queryAdmins = new ArrayList<>();
            switch (type) {
                case 1:
                    queryAdmins = tradeMapper.queryAdmins2(data.getString("id"), 7, "%Y-%m-%d");
                    break;
                case 2:
                    queryAdmins = tradeMapper.queryAdmins2(data.getString("id"), 30, "%Y-%m-%d");
                    break;
                case 3:
                    queryAdmins = tradeMapper.queryAdmins2(data.getString("id"), 120, "%Y-%m");
                    break;
                case 4:
                    queryAdmins = tradeMapper.queryAdmins2(data.getString("id"), 180, "%Y-%m");
                    break;
                case 5:
                    queryAdmins = tradeMapper.queryAdmins2(data.getString("id"), 360, "%Y-%m");
                    break;
                default:
                    queryAdmins = tradeMapper.queryAdmins2(data.getString("id"), 7, "%Y-%m-%d");
                    break;
            }
            WsEvent wsEvent = new WsEvent("adminDashboardQuerySelfRes", Result.success(queryAdmins));
            WebsocketManagerUtil.sendMessage(session, wsEvent);
            return;
        }
    }

    public void queryDevice(Session session, Manager manager, JSONObject data) {
        if (StringUtils.isBlank(data.getString("id")) || data.getInteger("type") == null) {
            WsEvent wsEvent = new WsEvent("adminDashboardQueryDeviceRes", Result.fail("参数错误"));
            WebsocketManagerUtil.sendMessage(session, wsEvent);
            return;
        }
        Integer type = data.getInteger("type");
        if (0 == type) {
            QueryWrapper<Trade> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("manager", manager.getId());
            queryWrapper.eq("device", data.getString("id"));
            List<QueryAdminVo> queryAdmins = tradeMapper.queryAdmins(queryWrapper);
            WsEvent wsEvent = new WsEvent("adminDashboardQueryDeviceRes", Result.success(queryAdmins));
            WebsocketManagerUtil.sendMessage(session, wsEvent);
        } else {
            List<QueryAdminVo> queryAdmins = new ArrayList<>();
            switch (type) {
                case 1:
                    queryAdmins = tradeMapper.queryAdmins3(data.getString("id"), 7, "%Y-%m-%d");
                    break;
                case 2:
                    queryAdmins = tradeMapper.queryAdmins3(data.getString("id"), 30, "%Y-%m-%d");
                    break;
                case 3:
                    queryAdmins = tradeMapper.queryAdmins3(data.getString("id"), 120, "%Y-%m");
                    break;
                case 4:
                    queryAdmins = tradeMapper.queryAdmins3(data.getString("id"), 180, "%Y-%m");
                    break;
                case 5:
                    queryAdmins = tradeMapper.queryAdmins3(data.getString("id"), 360, "%Y-%m");
                    break;
                default:
                    queryAdmins = tradeMapper.queryAdmins3(data.getString("id"), 7, "%Y-%m-%d");
                    break;
            }
            WsEvent wsEvent = new WsEvent("adminDashboardQueryDeviceRes", Result.success(queryAdmins));
            WebsocketManagerUtil.sendMessage(session, wsEvent);
            return;
        }
    }

    public void queryWine(Session session, Manager manager, JSONObject data) {
        if (StringUtils.isBlank(data.getString("id")) || data.getInteger("type") == null) {
            WsEvent wsEvent = new WsEvent("adminDashboardQueryWineRes", Result.fail("参数错误"));
            WebsocketManagerUtil.sendMessage(session, wsEvent);
            return;
        }
        Integer type = data.getInteger("type");
        if (0 == type) {
            QueryWrapper<Trade> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("manager", manager.getId());
            queryWrapper.eq("wine", data.getString("id"));
            List<QueryAdminVo> queryAdmins = tradeMapper.queryAdmins(queryWrapper);
            WsEvent wsEvent = new WsEvent("adminDashboardQueryWineRes", Result.success(queryAdmins));
            WebsocketManagerUtil.sendMessage(session, wsEvent);
        } else {
            List<QueryAdminVo> queryAdmins = new ArrayList<>();
            switch (type) {
                case 1:
                    queryAdmins = tradeMapper.queryAdmins4(data.getString("id"), 7, "%Y-%m-%d");
                    break;
                case 2:
                    queryAdmins = tradeMapper.queryAdmins4(data.getString("id"), 30, "%Y-%m-%d");
                    break;
                case 3:
                    queryAdmins = tradeMapper.queryAdmins4(data.getString("id"), 120, "%Y-%m");
                    break;
                case 4:
                    queryAdmins = tradeMapper.queryAdmins4(data.getString("id"), 180, "%Y-%m");
                    break;
                case 5:
                    queryAdmins = tradeMapper.queryAdmins4(data.getString("id"), 360, "%Y-%m");
                    break;
                default:
                    queryAdmins = tradeMapper.queryAdmins4(data.getString("id"), 7, "%Y-%m-%d");
                    break;
            }
            WsEvent wsEvent = new WsEvent("adminDashboardQueryWineRes", Result.success(queryAdmins));
            WebsocketManagerUtil.sendMessage(session, wsEvent);
            return;
        }
    }

    public void queryWorker(Session session, Manager manager, JSONObject data) {
        if (StringUtils.isBlank(data.getString("id")) || data.getInteger("type") == null) {
            WsEvent wsEvent = new WsEvent("adminDashboardQueryWorkerRes", Result.fail("参数错误"));
            WebsocketManagerUtil.sendMessage(session, wsEvent);
            return;
        }
        Integer type = data.getInteger("type");
        if (data.getInteger("type") == 0) {
            List<QueryAdminVo> adminVos = changeMapper.ChangeQueryWorkerForAdmin(manager.getId(), data.getString("id"));
            WsEvent wsEvent = new WsEvent("adminDashboardQueryWorkerRes", Result.success(adminVos));
            WebsocketManagerUtil.sendMessage(session, wsEvent);
            return;
        } else {
            List<QueryAdminVo> queryAdmins = new ArrayList<>();
            switch (type) {
                case 1:
                    queryAdmins = changeMapper.ChangeQueryWorkerForAdmin2(manager.getId(),data.getString("id"), 7, "%Y-%m-%d");
                    break;
                case 2:
                    queryAdmins = changeMapper.ChangeQueryWorkerForAdmin2(manager.getId(),data.getString("id"), 30, "%Y-%m-%d");
                    break;
                case 3:
                    queryAdmins = changeMapper.ChangeQueryWorkerForAdmin2(manager.getId(),data.getString("id"), 120, "%Y-%m");
                    break;
                case 4:
                    queryAdmins = changeMapper.ChangeQueryWorkerForAdmin2(manager.getId(),data.getString("id"), 180, "%Y-%m");
                    break;
                case 5:
                    queryAdmins = changeMapper.ChangeQueryWorkerForAdmin2(manager.getId(),data.getString("id"), 360, "%Y-%m");
                    break;
                default:
                    queryAdmins = changeMapper.ChangeQueryWorkerForAdmin2(manager.getId(),data.getString("id"), 7, "%Y-%m-%d");
                    break;
            }
            WsEvent wsEvent = new WsEvent("adminDashboardQueryWorkerRes", Result.success(queryAdmins));
            WebsocketManagerUtil.sendMessage(session, wsEvent);
            return;
        }
    }
}
